% Bayes function
% data: observed data object
%  X: R*C reaction flux rate data matrix
%  K: R*P reaction-pathway association binary matrix (prior knowledge)
% model: model object
%  tauEpsilon : random noise
%  B: R*P reaction-pathway continuous matrix
%  tauB : random noise
%  S: P*C pathway-condition matrix
%  Phi: P*P covariance matrix of S (pathway-pathway correlation)
%  hyperparameters:
%    alphaEpsilon: for the Gamma distribution
%    betaEpsilon: for the Gamma distribution
%    alphaB: for the Gamma distribution
%    betaB: for the Gamma distribution
%    nu: for a Wishart distribution
%    Psi: P*P matrix for Wishart distribution
% control: sampling paramters
%   iterNo: number of MCMC iterations
%   sampleNo: number of saved samples
%   burnNo: number of burn-in iterations
%   showIter: report every 'showIter' iterations
%   init: if true, the algorithm uses user-specified starting values
%
% Naruemon Pratanwanich - 2015 (modified from [1])
%% reference
% [1] Pratanwanich, Naruemon, and Pietro Li. "Pathway-based Bayesian inference of drugdisease interactions." Molecular bioSystems 10.6 (2014): 1538-1548.)
%%
function [model] = Bayes(data,model0,control)
% Setting internal parameters
[G,D] = size(data.X);
[~,P] = size(data.K);
% Initialisation
current.value.Z = data.K;
if control.init
    current.value.tauEpsilon = model0.tauEpsilon;
    current.value.B = model0.B;
    current.value.tauB = model0.tauB;
    current.value.Phi = model0.Phi;
    current.value.S = model0.S;
else
    current.value.tauEpsilon = 1;
    current.value.B = randn(G,P);
    current.value.tauB = 1;
    current.value.S = randn(P,D);
    current.value.Phi = cov(current.value.S);
end
%% Gibbs sampling
for t=1:control.iterNo
    if mod(t,control.showIter) == 0
        disp('Iteration ...')
        t
    end
    disp('Sample tau epsilon ...');
    [current.value.tauEpsilon] = sample_tauEpsilon();
    disp('Sample tau B ...');
    [current.value.tauB] = sample_tauB();
     disp('Sample B ...');
        for i=1:G
            current.value.B(i,data.K(i,:)==1) = sample_b_r(i);
            current.value.B(i,data.K(i,:)==0) = 0;
        end
    current.value.B = normalizeRow(current.value.B);
    disp('Sample Phi ...');
    current.value.Phi = sample_Phi();
    disp('Sample S ...');
    for i=1:D
        current.value.S(:,i) = sample_s_c(i);
    end
    %% Error analysis   
    RMSE = sqrt(sum(sum(((data.X - current.value.B*current.value.S).^2)))/(G*D));
    NRMSE = RMSE/(max(max(data.X))-min(min(data.X)));
    model.NRMSE(t) = NRMSE;
    
    %% Save samples
    disp('Save samples ...');
    if t>control.burnNo && mod(t-control.burnNo,control.sampleNo)==0
        sampleNo = (t-control.burnNo)/control.sampleNo;
        model.tauEpsilon(sampleNo) = current.value.tauEpsilon;
        model.Phi(:,:,sampleNo) = current.value.Phi;
        model.S(:,:,sampleNo) = current.value.S;
        model.B(:,:,sampleNo) = current.value.B;
        model.tauB(sampleNo) = current.value.tauB;
        model.Z(:,:,sampleNo) = current.value.Z;
    end
    
end
disp('Done Gibbs Sampling');

%% Output Manipulation
model.meanS = mean(model.S,3);
model.meanPhi = mean(model.Phi,3);
model.meanB = mean(model.B,3);
model.meanTauB = mean(model.tauB);
model.meanTauEpsilon = mean(model.tauEpsilon);
%% Functions for sampling
    function [tauEpsilon] = sample_tauEpsilon()
        alpha = model0.hyperparameters.alphaEpsilon + (G*D)/2;
        beta = model0.hyperparameters.betaEpsilon + 0.5*sum(sum((data.X-(current.value.B*current.value.S)).^2));
        tauEpsilon = gamrnd(alpha,1/beta);
    end

    function [b_r] = sample_b_r(r)
        subMatrix_S = current.value.S((data.K(r,:)==1),:);
        sub_P = size(subMatrix_S,1);
        Tau = current.value.tauB*ones(sub_P,sub_P) + current.value.tauEpsilon*(subMatrix_S*subMatrix_S');
        mu = Tau\eye(sub_P)*current.value.tauEpsilon*(subMatrix_S*data.X(r,:)');
        sigma = Tau\eye(sub_P);
        sigma = 0.5.*(sigma+sigma'); % make sure for symmetric        
        if sub_P>1 
        b_r = mvnrnd(mu,sigma);
        elseif sub_P==1
            b_r = normrnd(mu,sigma);
        else
            b_r = 0;
        end
    end
    function [tauB] = sample_tauB()
        alpha = model0.hyperparameters.alphaB + 0.5*sum(sum(current.value.Z));
        beta = model0.hyperparameters.betaB + 0.5*((norm(current.value.B,'fro'))^2);
        tauB = gamrnd(alpha,1/beta);
    end

    function [Phi] = sample_Phi()
        sumS = zeros(P,P);
        for d=1:D
            sumS = sumS + current.value.S(:,d)*current.value.S(:,d)';
        end
        nu = model0.hyperparameters.nu + D;
        Psi = inv(model0.hyperparameters.Psi) + sumS;
        Phi = wishrnd(inv(Psi),nu);
    end
    function [s_c] = sample_s_c(c)
        Phi = current.value.Phi + current.value.tauEpsilon*(current.value.B'*current.value.B);
        mu = (Phi\eye(P))*current.value.tauEpsilon*(current.value.B'*data.X(:,c));
        sigma = inv(Phi);
        s_c = mvnrnd(mu,sigma)';
    end


end